@@ -1,4 +1,4 @@
-# Copyright (c) 2008-2015 MetPy Developers.
+# Copyright (c) 2008,2015,2016 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains a collection of basic calculations.
@@ -16,7 +16,7 @@ import warnings
 
 import numpy as np
 
-from ..constants import g, omega, Rd
+from ..constants import G, g, me, omega, Rd, Re
 from ..package_tools import Exporter
 from ..units import atleast_1d, check_units, masked_array, units
 
@@ -257,6 +257,85 @@ def pressure_to_height_std(pressure):
     return (t0 / gamma) * (1 - (pressure / p0).to('dimensionless')**(Rd * gamma / g))
 
 
+@exporter.export
+@check_units('[length]')
+def height_to_geopotential(height):
+    r"""Compute geopotential for a given height.
+
+    Parameters
+    ----------
+    height : `pint.Quantity`
+        Height above sea level (array_like)
+
+    Returns
+    -------
+    `pint.Quantity`
+        The corresponding geopotential value(s)
+
+    Examples
+    --------
+    >>> from metpy.constants import g, G, me, Re
+    >>> import metpy.calc
+    >>> from metpy.units import units
+    >>> height = np.linspace(0,10000, num = 11) * units.m
+    >>> geopot = metpy.calc.height_to_geopotential(height)
+    >>> geopot
+    <Quantity([     0.           9817.46806283  19631.85526579  29443.16305888
+    39251.39289118  49056.54621087  58858.62446525  68657.62910064
+    78453.56156253  88246.42329545  98036.21574306], 'meter ** 2 / second ** 2')>
+
+    Notes
+    -----
+    Derived from definition of geopotential in [Hobbs2006]_ pg.14 Eq.1.8.
+
+    """
+    # Calculate geopotential
+    geopot = G * me * ((1 / Re) - (1 / (Re + height)))
+
+    return geopot
+
+
+@exporter.export
+def geopotential_to_height(geopot):
+    r"""Compute height from a given geopotential.
+
+    Parameters
+    ----------
+    geopotential : `pint.Quantity`
+        Geopotential (array_like)
+
+    Returns
+    -------
+    `pint.Quantity`
+        The corresponding height value(s)
+
+    Examples
+    --------
+    >>> from metpy.constants import g, G, me, Re
+    >>> import metpy.calc
+    >>> from metpy.units import units
+    >>> height = np.linspace(0,10000, num = 11) * units.m
+    >>> geopot = metpy.calc.height_to_geopotential(height)
+    >>> geopot
+    <Quantity([     0.           9817.46806283  19631.85526579  29443.16305888
+    39251.39289118  49056.54621087  58858.62446525  68657.62910064
+    78453.56156253  88246.42329545  98036.21574306], 'meter ** 2 / second ** 2')>
+    >>> height = metpy.calc.geopotential_to_height(geopot)
+    >>> height
+    <Quantity([     0.   1000.   2000.   3000.   4000.   5000.   6000.   7000.   8000.
+    9000.  10000.], 'meter')>
+
+    Notes
+    -----
+    Derived from definition of geopotential in [Hobbs2006]_ pg.14 Eq.1.8.
+
+    """
+    # Calculate geopotential
+    height = (((1 / Re) - (geopot / (G * me))) ** -1) - Re
+
+    return height
+
+
 @exporter.export
 @check_units('[length]')
 def height_to_pressure_std(height):
@@ -326,7 +405,7 @@ def add_height_to_pressure(pressure, height):
         The corresponding pressure value for the height above the pressure level
 
     See Also
-    -----
+    --------
     pressure_to_height_std, height_to_pressure_std, add_pressure_to_height
 
     """
@@ -354,7 +433,7 @@ def add_pressure_to_height(height, pressure):
         The corresponding height value for the pressure above the height level
 
     See Also
-    -----
+    --------
     pressure_to_height_std, height_to_pressure_std, add_height_to_pressure
 
     """
